import { RequestCore, TaskManagerCoreConfig, SharedConfigs, RequestConfigs, RequestEventHandlersType } from './request-core.mjs';
export { RequestConfigBase, RequestDataRecordType, RequestStatus, TaskManagerInterceptorsType } from './request-core.mjs';
import './cache-core.mjs';
import 'idb-keyval';
import './encode-core.mjs';

/**
 * TaskManagerCore manages the tasks and triggers events based on the task's lifecycle.
 * The onCache, onRequest, onSuccess, onError, and onFinally events are triggered based on the task's lifecycle.
 *
 * @template Task - A function type that returns a Promise.
 * @template TaskError - The type of error that can be thrown by the task.
 */
declare class TaskManagerCore<Task extends (...args: any) => Promise<any>, TaskError = any, Meta = Record<string, any>> extends RequestCore<Task, TaskError, Meta> {
    constructor(config: TaskManagerCoreConfig<Task, TaskError, Meta>);
    /**
     * Initiates a request with the given parameters.
     *
     * @param {...Parameters<Task>} parameters - The parameters to pass to the task.
     * @returns {Promise<any>} - The result of the task.
     */
    request(...parameters: Parameters<Task>): Promise<{
        data?: Awaited<ReturnType<Task>> | null | undefined;
        error?: TaskError | undefined;
        isError?: boolean;
        createdAt: number;
        updatedAt: number;
        hash: string;
        parameters: Parameters<Task>;
        meta: Meta;
        isPrevented?: boolean;
    } & SharedConfigs>;
    /**
     * Configures the request with the given configuration.
     *
     * @param {RequestConfigs<Task, TaskError, Meta>} config - The configuration for the request.
     * @returns {Object} - An object containing a request function.
     */
    config(config: RequestConfigs<Task, TaskError, Meta>): {
        request: (...parameters: Parameters<Task>) => Promise<any>;
    };
    /**
     * Invalidates the current request.
     * it will ignore any blocker logics and re-fetch the data.
     *
     * @param {Omit<RequestEventHandlersType<Task, TaskError,Meta>, "onCache">} [config={}] - The configuration for the request.
     * @returns {Promise<any>} - The result of the invalidated task.
     */
    invalidate(config?: Omit<RequestEventHandlersType<Task, TaskError, Meta>, "onCache">): Promise<any>;
    /**
     * Pre-processes the request with the given parameters.
     * It only affects the cache and does not change the current request parameters.
     *
     * @param {...Parameters<Task>} parameters - The parameters to pass to the task.
     * @returns {Promise<any>} - The result of the pre-processed task.
     */
    preProcess(...parameters: Parameters<Task>): Promise<{
        data?: Awaited<ReturnType<Task>> | null | undefined;
        error?: TaskError | undefined;
        isError?: boolean;
        createdAt: number;
        updatedAt: number;
        hash: string;
        parameters: Parameters<Task>;
        meta: Meta;
        isPrevented?: boolean;
    } & SharedConfigs>;
}
declare function taskManager<Task extends (...args: any) => Promise<any>, TaskError = any, Meta = Record<string, any>>(config: TaskManagerCoreConfig<Task, TaskError, Meta>): TaskManagerCore<Task, TaskError, Meta>;

export { RequestConfigs, RequestEventHandlersType, SharedConfigs, TaskManagerCore, TaskManagerCoreConfig, taskManager };
